﻿using Eto.Drawing;
using Eto.Forms;
using Rhino.PlugIns;
using Rhino.UI;
using System;

namespace CustomRhinoObjectTest
{
    public class CustomRhinoObjectTestPlugIn : Rhino.PlugIns.PlugIn

    {
        public CustomRhinoObjectTestPlugIn()
        {
            Instance = this;
        }
         
        public static CustomRhinoObjectTestPlugIn Instance
        {
            get; private set;
        }
        protected override LoadReturnCode OnLoad(ref string errorMessage)
        {
            Panels.RegisterPanel(this, typeof(RhinoPanel), "CustomRhinoObjectTest", null);

            return base.OnLoad(ref errorMessage);
        }
    }

    [System.Runtime.InteropServices.Guid("e4848f77-fafe-4c93-89f2-589d4ec73d17")]
    public class RhinoPanel : Panel, IPanel
    {
        #region Fields
        private readonly uint _document_sn;  
        #endregion

        #region Properties
        public static Guid Id => typeof(RhinoPanel).GUID;
        public DynamicLayout Container { get; }
        public static RhinoPanel Instance { get; private set; }

        public Entity ActiveEntity { get; private set; }
        #endregion

        #region Constructors
        public RhinoPanel() { Instance = this; }
        public RhinoPanel(uint document_sn)
        {
            _document_sn = document_sn;
            Instance = this;
           
            Container = new DynamicLayout();
            Container.Padding = new Padding(3);
            Container.Spacing = new Size(0, 6);
            Content = Container;
        }
        #endregion

        #region Methods 
        Slider sliderInflation;
        EnumDropDown<Colors> dropDownColor;
        private bool _sliderChanged;
        private bool _dropDownChanged;
        public void SetEntity(Entity entity)
        {
            ActiveEntity = entity;
            if (entity == null)
            {
                Container.Clear();
            }
            else
            {
                sliderInflation = new Slider()
                { 
                    MinValue = 0,
                    MaxValue = 100,
                    Value = (int)((ActiveEntity.Inflation / 8.0) * 100),
                    Orientation = Orientation.Horizontal,
                    SnapToTick = false,
                    TickFrequency = 1, 
                    };
                sliderInflation.ValueChanged += SliderInflation_ValueChanged;
                entity.PropertyChanged += Entity_PropertyChanged;
                Container.AddSeparateRow(xscale: true, spacing: new Size(0,4), controls: 
                    new Control[] { new Label() { Text = "Inflation" }, sliderInflation });

                dropDownColor = new EnumDropDown<Colors>() { 
                SelectedValue = ActiveEntity.Color
                }; 
                dropDownColor.SelectedValueChanged += DropDownColor_SelectedValueChanged;
               Container.AddSeparateRow(xscale: true, spacing: new Size(0, 4), controls:
                   new Control[] { new Label() { Text = "Color" }, dropDownColor, null });

                Container.AddRow(null);
            }
            Container.Create();
        }

        private void DropDownColor_SelectedValueChanged(object sender, EventArgs e)
        {
            _dropDownChanged = true;
            ActiveEntity.Color = ((EnumDropDown<Colors>)sender).SelectedValue;
            _dropDownChanged = false; 
        }

        private void SliderInflation_ValueChanged(object sender, EventArgs e)
        {
            _sliderChanged = true;
            ActiveEntity.Inflation = 8 * (((Slider)sender).Value / 100.0);
            _sliderChanged = false;
      
        }

        private void Entity_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Inflation")
            {
                if (!_sliderChanged)
                {
                    sliderInflation.Value = (int)((ActiveEntity.Inflation / 8.0) * 100);
                }
            }
            else if (e.PropertyName == "Color")
            {
                if (!_dropDownChanged)
                {
                    dropDownColor.SelectedValue = ActiveEntity.Color;
                }
            }
            Rhino.RhinoDoc.ActiveDoc.Views.Redraw();
        }



        #region IPanel 
        public void PanelShown(uint documentSerialNumber, ShowPanelReason reason)
        {
            // Called when the panel tab is made visible, in Mac Rhino this will happen
            // for a document panel when a new document becomes active, the previous
            // documents panel will get hidden and the new current panel will get shown. 
        }

        public void PanelHidden(uint documentSerialNumber, ShowPanelReason reason)
        {
            // Called when the panel tab is hidden, in Mac Rhino this will happen
            // for a document panel when a new document becomes active, the previous
            // documents panel will get hidden and the new current panel will get shown. 
        }

        public void PanelClosing(uint documentSerialNumber, bool onCloseDocument)
        {
            // Called when the document or panel container is closed/destroyed 
        }
        #endregion

        public static void OpenPanel()
        {
            if (Rhino.UI.Panels.IsPanelVisible(Id))
            {
                return;
            }
            Rhino.UI.Panels.OpenPanel(Id);
        }
        #endregion 
    }
}