﻿using System;
using System.Collections.Generic;
using Rhino;
using Rhino.Commands;
using Rhino.DocObjects;
using Rhino.Geometry;
using Rhino.Input;
using Rhino.Input.Custom;

namespace CustomRhinoObjectTest
{
    public class EntityCurveCommand : Command
    {
        public static EntityCurveCommand Instance { get; private set; }
        public override string EnglishName { get { return "EntityCurve"; } }

        public EntityCurveCommand()
        { 
            Instance = this;
        }
          
        protected override Result RunCommand(RhinoDoc doc, RunMode mode)
        {
            var result = RhinoGet.GetOneObject("Select curve", false, ObjectType.Curve, out ObjRef objRef);
            if (result != Result.Success)
                return result; 
            var entity = new EntityCurve() { Inflation = 5, Value = objRef.Curve() };
            var rhObj = entity.ToRhinoObject() as RhinoEntityCurve;
            // This remove rhObj attributes
            //doc.Objects.Replace(objRef, rhObj); 
            doc.Objects.Delete(objRef, true);
            doc.Objects.AddRhinoObject(rhObj, null);
          
            doc.Views.Redraw();
            return Result.Success;
        }
    }

    public class ModifyEntityCurveCommand : Command
    {
        public static ModifyEntityCurveCommand Instance { get; private set; }
        public override string EnglishName { get { return "EntityCurveModify"; } }

        public ModifyEntityCurveCommand()
        {
            Instance = this;
        }

        protected override Result RunCommand(RhinoDoc doc, RunMode mode)
        {
            var result = RhinoGet.GetOneObject("Select entity curve", false, ObjectType.Curve, out ObjRef objRef);
            if (result != Result.Success)
                return result;

            if(objRef.Object() is RhinoEntityCurve rhObj)
            {
                var entity = rhObj.Entity; 
                var go = new GetOption();
                go.AcceptNothing(true);
                go.SetCommandPrompt("Select param");
                go.SetDefaultString("");
                var optInflation = new OptionDouble(entity.Inflation);
                go.AddOptionDouble("Inflation", ref optInflation);
                var colors = (Colors[])Enum.GetValues(typeof(Colors));
                var optColor = go.AddOptionEnumList("Color", entity.Color, colors);

                while (true)
                {
                    var res = go.Get();
                    if (res == GetResult.Option)
                    {
                        if(go.OptionIndex() == optColor)
                        {
                            entity.Color = colors[go.Option().CurrentListOptionIndex]; 
                        } 
                        entity.Inflation = optInflation.CurrentValue;
                        doc.Views.Redraw();
                        continue;
                    } 
                    else if (res != GetResult.Nothing)
                    {
                        return Result.Cancel;
                    }

                    break;
                } 
            }
            else
            {
                RhinoApp.WriteLine("Selected curve is not an entity");
            }
              
            return Result.Success;
        }
    }
}
